package edu.uoc.helpDesk.entities;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * Clase que almacena una entidad de datos de tipo Incident o incidente.
 * 
 * @author Miguel Souto Bartolomé
 * @version 1.0 13/01/2014
 * 
 */
@Entity
public class Incident implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * El método de creación.
	 */
	public Incident() {
		super();
	}

	/**
	 * El identificador del incidente.
	 */
	@Id
	@GeneratedValue
	private int incidentId;

	/**
	 * El identificador del trabajador. Clave foránea al identificador del
	 * trabajador de la clase Worker.
	 */
	@OneToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "workerId")
	private Worker workerId;

	/**
	 * El identificador del ingeniero. Clave foránea al identificador del
	 * ingeniero de la clase Engineer.
	 */
	@OneToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "engineerId", referencedColumnName = "workerId")
	private Engineer engineerId;

	/**
	 * La descripción de la incidencia dada por el trabajador.
	 */
	private String incidentDescription;

	/**
	 * La solución a la incidencia dada por el ingeniero.
	 */
	private String incidentSolution;

	/**
	 * Fecha y hora de apertura de la incidencia.
	 */
	@Temporal(TemporalType.TIMESTAMP)
	private java.util.Date incidentOpen;

	/**
	 * Fecha y hora de cierre de la incidencia.
	 */
	@Temporal(TemporalType.TIMESTAMP)
	private java.util.Date incidentClose;

	/**
	 * Estado de la incidencia. Puede adoptar varios valores:</p> 0 - no
	 * asignada</br> 1 - asignada</br> 2 - en resolución</br> 3 - cerrada (o
	 * solucionada.</br> 4 - reabierta (el administrador no ha aceptado la
	 * solución propuesta).
	 */
	private int incidentState;

	/**
	 * Obtiene el identificador de la incidencia.
	 * 
	 * @return El identificador de la incidencia.
	 */
	public int getIncidentId() {
		return incidentId;
	}

	/**
	 * Asigna el identificador de la incidencia.
	 * 
	 * @param incidentId
	 *            El identificador de la incidencia.
	 */
	public void setIncidentId(int incidentId) {
		this.incidentId = incidentId;
	}

	/**
	 * Obtiene el trabajador que ha notificado la incidencia. Se obtiene
	 * directamente el momento de realizar la petición (opción EAGER del método
	 * Fetch).
	 * 
	 * @return El trabajador.
	 */
	public Worker getWorkerId() {
		return workerId;
	}

	/**
	 * Asigna el trabajador.
	 * 
	 * @param workerId El trabajador.
	 */
	public void setWorkerId(Worker workerId) {
		this.workerId = workerId;
	}

	/**
	 * Obtiene el ingeniero encargado de solucionar la incidencia. Se obtiene
	 * directamente el momento de realizar la petición (opción EAGER del método
	 * Fetch).
	 * 
	 * @return El ingeniero.
	 */
	public Engineer getEngineerId() {
		return engineerId;
	}

	/**
	 * Asigna el ingeniero.
	 * 
	 * @param engineerId El ingeniero.
	 */
	public void setEngineerId(Engineer engineerId) {
		this.engineerId = engineerId;
	}

	/**
	 * Obtiene la descripción de la incidencia.
	 * 
	 * @return La descripción de la incidencia.
	 */
	public String getIncidentDescription() {
		return incidentDescription;
	}

	/**
	 * Asigna la descripción de la incidencia.
	 * 
	 * @param incidentDescription La descripción de la incidencia.
	 */
	public void setIncidentDescription(String incidentDescription) {
		this.incidentDescription = incidentDescription;
	}

	/**
	 * Obtiene la solución de la incidencia.
	 * 
	 * @return La solución de la incidencia.
	 */
	public String getIncidentSolution() {
		return incidentSolution;
	}

	/**
	 * Asigna la solución de la incidencia.
	 * 
	 * @param incidentSolution La solución de la incidencia.
	 */
	public void setIncidentSolution(String incidentSolution) {
		this.incidentSolution = incidentSolution;
	}

	/**
	 * Obtiene la fecha de apertura de la incidencia.
	 * 
	 * @return La fecha de apertura de la incidencia.
	 */
	public java.util.Date getIncidentOpen() {
		return incidentOpen;
	}

	/**
	 * Asigna la fecha de apertura de la incidencia.
	 * 
	 * @param incidentOpen La fecha de apertura de la incidencia.
	 */
	public void setIncidentOpen(java.util.Date incidentOpen) {
		this.incidentOpen = incidentOpen;
	}

	/**
	 * Obtiene la fecha de cierre de la incidencia.
	 * 
	 * @return La fecha de cierre de la incidencia.
	 */
	public java.util.Date getIncidentClose() {
		return incidentClose;
	}

	/**
	 * Asigna la fecha de cierre de la incidencia.
	 * 
	 * @param incidentClose La fecha de cierre de la incidencia.
	 */
	public void setIncidentClose(java.util.Date incidentClose) {
		this.incidentClose = incidentClose;
	}

	/**
	 * Obtiene el estado de la incidencia.
	 * 
	 * @return El estado de la incidencia.
	 */
	public int getIncidentState() {
		return incidentState;
	}

	/**
	 * Asigna el estado de la incidencia.
	 * 
	 * @param incidentState El estado de la incidencia.
	 */
	public void setIncidentState(int incidentState) {
		this.incidentState = incidentState;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "Incident [incidentId=" + incidentId + ", workerId=" + workerId
				+ ", engineerId=" + engineerId + ", incidentDescription="
				+ incidentDescription + ", incidentSolution="
				+ incidentSolution + ", incidentOpen=" + incidentOpen
				+ ", incidentClose=" + incidentClose + ", incidentState="
				+ incidentState + "]";
	}
}
