package edu.uoc.myst.auxiliary;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Logger;
import java.lang.reflect.Array;
import java.lang.reflect.Method;

import edu.uoc.myst.core.ContextObject;
import edu.uoc.myst.exception.ConversionException;

/**
 * Clase que provee un conjunto de herramientas para facilitar el tratamiento y
 * la conversin de objetos de tipo {@link edu.uoc.myst.form.Form Form}.</p>
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class FormTools {

	static Logger logger = Logger.getLogger(FormTools.class.getName());

	/**
	 * Mtodo que carga los elementos de un mapa en un formulario de tipo
	 * {@link edu.uoc.myst.form.Form Form}.</p>
	 * 
	 * El Form debe definir atributos para cada elemento del mapa, con el mismo
	 * nombre y con un mtodo setter asociado.</p>
	 * 
	 * Las posibles excepciones se almacenarn el atributo exceptions del
	 * contexto de la peticin HTTP, accesible a travs del mtodo
	 * {@link edu.uoc.myst.core.ContextObject#getExceptions() getExceptions}</p>
	 * 
	 * @param form
	 *            El formulario.
	 * @param map
	 *            El mapa.
	 */
	public static void mapToForm(Object form, Map<?, ?> map,
			ContextObject context) {

		Map.Entry<?, ?> entry;
		String name;
		Method method = null;
		Class<?> type = null;
		Object result = null;
		Method[] methods;
		String[] stringArray;
		Iterator<?> entries = map.entrySet().iterator();

		while (entries.hasNext()) {
			entry = (Map.Entry<?, ?>) entries.next();

			// El nombre del atributo
			name = (String) entry.getKey();
			if (name == null) {
				continue;
			}
			// Obtiene el nombre del mtodo setter asociado al atributo
			String methodName = ("set"
					+ name.substring(0, 1).toUpperCase(Locale.ENGLISH) + name
					.substring(1, name.length()));
			logger.info("Nombre del mtodo: " + methodName);

			try {
				/*
				 * Obtiene el mtodo setter asociado entre los mtodos del
				 * formulario.
				 */
				methods = form.getClass().getMethods();
				for (int i = 0; i < methods.length; i++) {
					if (methods[i].getName().equals(methodName)) {
						type = methods[i].getParameterTypes()[0];
						break;
					}
				}
				method = form.getClass().getMethod(methodName, type);

				/*
				 * En caso de excepcin se aade al contexto y se contina la
				 * inspeccin del resto de atributos.
				 */
			} catch (NoSuchMethodException e) {
				logger.info("El mtodo " + methodName
						+ " no est definido en la clase "
						+ form.getClass().getName());
				context.addException(e);
				continue;

			} catch (SecurityException e) {
				logger.info("El mtodo " + methodName + " de la clase "
						+ form.getClass() + " debe ser pblico.");
				context.addException(e);
				continue;
			}

			if (isSetter(method)) {
				/*
				 * El mtodo es un setter, se adapta el valor del mapa y se le
				 * pasa como parmetro al setter.
				 */

				stringArray = (String[]) entry.getValue();
				try {
					result = convert(stringArray, type);

					/*
					 * En caso de excepcin se aade al contexto y se contina
					 * la inspeccin del resto de atributos.
					 */
				} catch (ConversionException e) {
					context.addException(e);
					continue;
				} catch (Exception e) {
					logger.info("Acceso incorrecto al array " + stringArray);
					context.addException(e);
					continue;
				}
				try {
					// Se llama al setter.
					method.invoke(form, result);

					// En caso de error se aade al contexto.
				} catch (Exception e) {
					logger.info("El mtodo " + methodName + "("
							+ type.getName() + ")" + " de la clase "
							+ form.getClass().getName() + " no es accesible.");
					context.addException(e);
				}
			}
		}
	}

	/**
	 * Guarda los elementos de un formulario de tipo
	 * {@link edu.uoc.myst.form.Form Form} en un {@link Map}.
	 * 
	 * @param form
	 *            El formulario.
	 * @return El mapa.
	 * @throws Exception
	 * @throws NotInvokableException
	 */
	public static Map<String, Object> formToMap(Object form) throws Exception {

		Map<String, Object> formMap = new HashMap<String, Object>();
		String elementName;
		String methodName;
		String formClass = form.getClass().getName();
		String declaringClass;
		Object result = null;

		logger.info("Clase a convertir: " + formClass);
		Method[] methods = form.getClass().getDeclaredMethods();

		for (int i = 0; i < methods.length; i++) {

			// Se recorren los mtodos del form para invocar los getters
			methodName = methods[i].getName();
			declaringClass = methods[i].getDeclaringClass().getName();
			elementName = (methodName.substring(3, 4).toLowerCase(
					Locale.ENGLISH) + methodName.substring(4,
					methodName.length()));

			if (isGetter(methods[i]) && (!methods[i].isSynthetic())
					&& declaringClass.equals(formClass)) {
				/*
				 * El mtodo inspeccionado es un getter, se intenta invocar para
				 * obtener el parmetro.
				 */
				logger.info("Obtener parmetro " + elementName + " con "
						+ methodName);
				try {

					result = methods[i].invoke(form, (Object[]) null);

					/*
					 * En caso de error se detiene el proceso y se lanza la
					 * excepcin.
					 */
				} catch (Exception e) {
					logger.info("El mtodo " + methodName + " de la clase "
							+ form.getClass().getName() + " no es accesible.");
					throw e;
				}
				// El proceso ha sido correcto, se guarda el elemento en el Map.
				formMap.put(elementName, result);
			}
		}
		return formMap;
	}

	/**
	 * Comprueba que el objeto sea conforme a la especificacin de
	 * {@link edu.uoc.myst.form.Form Form}.
	 * 
	 * @param object
	 *            El objeto.
	 * @return true si es conforme, false si no lo es.
	 */
	public static boolean isForm(Object object) {
		boolean result = false;

		/*
		 * Se compureba que el objeto sea de tipo primitivo, o bien un Array, de
		 * tipo Collection o de tipo Map
		 */
		if (isPrimitive(object) || object.getClass().isArray()
				|| (object instanceof Collection<?>)
				|| (object instanceof Map<?, ?>)) {
			result = false;
			return result;
		}

		/*
		 * Se comprueba que el objeto defina algn mtodo getter.
		 */
		Method[] methods = object.getClass().getDeclaredMethods();
		for (int i = 0; i < methods.length; i++) {
			if (isGetter(methods[i])) {
				result = true;
				break;
			}
		}
		return result;
	}

	/**
	 * Comprueba si el dato pasado por parmero es de tipo primitivo. Acepta
	 * datos numricos, y tipos Date, String y Boolean.
	 * 
	 * @param value
	 *            El dato.
	 * @return true si el dato es primitivo, false si no lo es.
	 */
	public static boolean isPrimitive(Object value) {
		if (isNumber(value) || Date.class.isAssignableFrom(value.getClass())
				|| value.getClass().equals(String.class)
				|| value.getClass().equals(Boolean.class)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Convierte un objeto en su clase envoltorio o wrapper primitivo. En caso
	 * de ser un array lo convierte en un array cuyas componentes son del tipo
	 * envoltorio primitivo adecuado.</p>
	 * 
	 * @param object
	 *            El objeto.
	 * @param wrapper
	 *            El envoltorio.
	 * @return El objeto convertido en envoltorio.
	 * @throws ConversionException
	 * @throws IllegalArgumentException
	 * @throws ArrayIndexOutOfBoundsException
	 * @see #adapt(String, Class)
	 */
	private static Object convert(String[] object, Class<?> wrapper)
			throws ArrayIndexOutOfBoundsException, IllegalArgumentException,
			ConversionException {

		if (wrapper.isArray()) {
			// Es un array, se convierten todas sus componentes individualmente.
			Object result = Array.newInstance(wrapper.getComponentType(),
					object.length);
			for (int i = 0; i < object.length; i++) {
				Array.set(result, i,
						adapt(object[i], wrapper.getComponentType()));
			}
			return result;
		} else
			// El objeto no es una coleccin, se convierte.
			return adapt(object[0], wrapper);
	}

	/**
	 * Convierte un tipo de dato primitivo en su envoltorio o wrapper
	 * equivalente.</p>
	 * 
	 * @param primitive
	 *            El dato primitivo.
	 * @param wrapper
	 *            El envoltorio.
	 * @return El dato representado como envoltorio.
	 * @throws ConversionException
	 */
	private static Object adapt(String primitive, Class<?> wrapper)
			throws ConversionException {
		try {
			if (wrapper.equals(int.class)) {
				return (Integer.parseInt(primitive));
			} else if (wrapper.equals(double.class)) {
				return (Double.parseDouble(primitive));
			} else if (wrapper.equals(long.class)) {
				return (Long.parseLong(primitive));
			} else if (wrapper.equals(float.class)) {
				return (Float.parseFloat(primitive));
			} else if (wrapper.equals(byte.class)) {
				return (Byte.parseByte(primitive));
			} else if (wrapper.equals(short.class)) {
				return (Short.parseShort(primitive));
			} else if (wrapper.equals(char.class)) {
				if (primitive.length() == 1) {
					return (primitive.charAt(0));
				} else {
					throw new Exception();
				}
			} else if (wrapper.equals(boolean.class)) {
				return (Boolean.parseBoolean(primitive));
			} else if (wrapper.equals(String.class)) {
				return primitive;
			} else {
				throw new Exception();
			}
			
		} catch (Exception e) {
			// el objeto no es de tipo primitivo, se lanza una excepcin.
			logger.info("El objeto " + primitive
					+ " no puede ser convertido a  " + wrapper.getName());
			throw new ConversionException(primitive, wrapper.getName());
		}
	}

	/**
	 * Comprueba que un mtodo cuyo nombre empieza por "get" sea realmente un
	 * getter.
	 * 
	 * @param method
	 *            El mtodo.
	 * @return true si es un getter, false si no lo es.
	 */
	private static boolean isGetter(Method method) {
		// El nombre del mtodo debe empezar por "get".
		if (!method.getName().startsWith("get"))
			return false;
		
		// Debe admitir un nmero de parmetros igual a cero.
		if (method.getParameterTypes().length != 0)
			return false;
		
		// Debe devolver un objeto de algn tipo.
		if (void.class.equals(method.getReturnType()))
			return false;
		return true;
	}

	/**
	 * Comprueba que un mtodo cuyo nombre empieza por "set" sea realmente un
	 * setter.
	 * 
	 * @param method
	 *            El mtodo.
	 * @return true si es un setter, false si no lo es.
	 */
	private static boolean isSetter(Method method) {
		// El nombre del mtodo debe empezar por "get".
		if (!method.getName().startsWith("set"))
			return false;
		
		// Debe admitir un nmero de parmetros igual a uno.
		if (method.getParameterTypes().length != 1)
			return false;
		return true;
	}

	/**
	 * Comprueba si el dato pasado por parmero es numrico.
	 * 
	 * @param value
	 *            El dato.
	 * @return true si es numrico, false si no lo es
	 */
	private static boolean isNumber(Object value) {
		try {
			Double.parseDouble(value.toString());
		} catch (Exception e) {
			return false;
		}
		return true;
	}
}
