package edu.uoc.myst.auxiliary;

import java.util.Map;
import java.util.logging.Logger;
import java.util.regex.Pattern;

import edu.uoc.myst.exception.NoPathException;
import edu.uoc.myst.exception.WrongPatternException;

/**
 * Clase que proporciona herramientas para el tratamiento de expresiones
 * regulares.
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class RegExTools {

	static Logger logger = Logger.getLogger(RegExTools.class.getName());

	/**
	 * Obtiene el ltimo elemento de una ruta dentro de un mapa
	 * proporcionado.</p>
	 * 
	 * La ruta proporcionada es navegable, y se puede acceder a cualquier nivel
	 * de profundidad. Por ejemplo, para acceder al atributo ptalo de un
	 * elemento flor la ruta sera:</p>
	 * 
	 * "flor.petalo"</p>
	 * 
	 * Si ptalo fuera un array con varios elementos, tambin se pueden acceder
	 * individualmente, como a su cuarto elemento:</p>
	 * 
	 * "flor.petalo[3]"</p>
	 * 
	 * Y si en cada ptalo hubiera una mosca:</p>
	 * 
	 * "flor.petalo[3].mosca"</p>
	 * 
	 * @param element
	 *            La ruta del elemento.
	 * @param map
	 *            El mapa donde buscarlo.
	 * @return El elemento.
	 * @throws WrongPatternException
	 * @throws NoPathException
	 */
	@SuppressWarnings("unchecked")
	public static Object getElement(String element, Object map)
			throws WrongPatternException, NoPathException {

		Map<String, Object> newMap = null;
		logger.info("Recuperar ruta " + element);

		try {
			if (Pattern.matches("[a-zA-Z_][a-zA-Z0-9_\\-]*\\..+", element)) {
				/*
				 * La ruta es un mapa con atributos. Se obtiene el mapa y se
				 * sigue inspeccionando el resto de la ruta.
				 */
				int index = element.indexOf(".");
				String firstElement = element.substring(0, index);
				String restElements = element.substring(index + 1,
						element.length());
				newMap = (Map<String, Object>) ((Map<String, Object>) map)
						.get(firstElement);
				return (getElement(restElements, newMap));

			} else if (Pattern.matches(
					"[a-zA-Z_][a-zA-Z0-9_\\-]*\\[[1-9]*[0-9]\\].+", element)) {
				/*
				 * La ruta es un vector con atributos en sus elementos. El
				 * vector est categorizado por los parntesis.
				 */
				int indexOpen = element.indexOf("[");
				int indexClose = element.indexOf("]");
				Object[] array = (Object[]) ((Map<String, Object>) map)
						.get(element.substring(0, indexOpen));
				String restElements;

				if (element.charAt(indexClose + 1) == '.') {
					/*
					 * El caracter siguiente al vector es un punto, por lo que
					 * se obtiene un mapa.
					 */
					newMap = (Map<String, Object>) array[Integer
							.parseInt(element.substring(indexOpen + 1,
									indexClose))];
					restElements = element.substring(indexClose + 2,
							element.length());
					return (getElement(restElements, newMap));

				} else {
					/*
					 * El caracter siguiente al vector es el principio de un
					 * vector, por lo que se obtiene un nuevo vector.
					 */
					Object[] newArray = (Object[]) array[Integer
							.parseInt(element.substring(indexOpen + 1,
									indexClose))];
					restElements = element.substring(indexClose + 1,
							element.length());
					return (getElement(restElements, newArray));
				}

			} else if (Pattern.matches(
					"[a-zA-Z_][a-zA-Z0-9_\\-]*\\[[1-9]*[0-9]\\]", element)) {
				/*
				 * El elemento est en una posicin de un vector. La ruta
				 * termina en unos parntesis con un elemento dentro.
				 */
				int indexOpen = element.indexOf("[");
				Object[] array = (Object[]) ((Map<String, Object>) map)
						.get(element.substring(0, indexOpen));
				return (array[Integer.parseInt(element.substring(indexOpen + 1,
						element.length() - 1))]);

			} else if (Pattern.matches("\\[[1-9]*[0-9]\\].+", element)) {
				// La ruta describe la posicin de un vector.
				int indexClose = element.indexOf("]");
				logger.info("Vector: " + element);
				Object[] array = (Object[]) map;
				String restElements;

				if (element.charAt(indexClose + 1) == '.') {
					/*
					 * El caracter siguiente al vector es un punto, por lo que
					 * se obtiene un mapa.
					 */
					newMap = (Map<String, Object>) array[Integer
							.parseInt(element.substring(1, indexClose))];
					restElements = element.substring(indexClose + 2,
							element.length());
					return (getElement(restElements, newMap));

				} else {
					/*
					 * El caracter siguiente al vector es el principio de un
					 * vector, por lo que se obtiene un nuevo vector.
					 */
					Object[] newArray = (Object[]) array[Integer
							.parseInt(element.substring(1, indexClose))];
					restElements = element.substring(indexClose + 1,
							element.length());
					return (getElement(restElements, newArray));
				}

			} else if (Pattern.matches("\\[[1-9]*[0-9]\\]", element)) {
				/*
				 * El elemento est en una posicin de un vector, se devuelve el
				 * elemento.
				 */
				Object[] array = (Object[]) map;
				return (array[Integer.parseInt(element.substring(1,
						element.length() - 1))]);

			} else if (Pattern.matches("[a-zA-Z_][a-zA-Z0-9_\\-]*", element)) {
				/*
				 * El elemento est en un mapa, se devuelve el
				 * elemento.
				 */
				return (((Map<String, Object>) map).get(element));

			} else {
				// La ruta no respeta la sintaxis definida, no puede tratarse.
				logger.info("El elemento " + element
						+ " no respeta la sintaxis de una ruta.");
				throw new WrongPatternException(element);
			}
			
		} catch (ClassCastException e) {
			// La ruta es falsa, no existe ne la lgica de negocio
			logger.info("La ruta " + element + " no existe.");
			throw new NoPathException(element);
		}
	}
}
