package edu.uoc.myst.command;

import edu.uoc.myst.core.ContextObject;
import edu.uoc.myst.exception.MystException;
import edu.uoc.myst.view.ViewHelper;

/**
 * Interfaz que permite definir el comando asociado a una peticin HTTP.</p>
 * 
 * Todo comando que utilice el framework Myst puede implementar esta clase,
 * extender la clase abstracta {@link AbstractCommand}, o ser un objeto POJO que
 * respete las siguientes especificaciones:</p>
 * 
 * <ul>
 * <li>Si utiliza un formulario {@link edu.uoc.myst.form.Form Form}, debe
 * definir el mtodo pblico {@link #setFormBean(Object) setFormBean}.</li>
 * <li>Si utiliza un {@link edu.uoc.myst.view.ViewHelper ViewHelper} para
 * facilitar las vistas, debe definir el mtodo pblico
 * {@link #setViewHelper(ViewHelper) setViewHelper}.</li>
 * <li>Si adems quiere tener acceso al contexto de la peticin HTTP, debe
 * definir el mtodo pblico {@link #setContextObject(ContextObject).
 * setContextObject}</li>
 * </ul>
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public interface Command {

	/**
	 * Asigna el formulario.
	 * 
	 * @param formBean
	 *            El formulario.
	 */
	public void setFormBean(Object formBean);

	/**
	 * Devuelve el formulario.
	 * 
	 * @return El formulario.
	 */
	public Object getFormBean();

	/**
	 * Asigna el ViewHelper.
	 * 
	 * @param viewHelper
	 *            El ViewHelper.
	 */
	public void setViewHelper(ViewHelper viewHelper);

	/**
	 * Devuelve el ViewHelper.
	 * 
	 * @return El ViewHelper.
	 */
	public ViewHelper getViewHelper();

	/**
	 * Asigna el contexto de la peticin HTTP.
	 * 
	 * @param contextObject
	 *            El contexto.
	 */
	public void setContextObject(ContextObject contextObject);

	/**
	 * Devuelve el contexto de la peticin HTTP.
	 * 
	 * @return El contexto.
	 */
	public ContextObject getContextObject();

	/**
	 * Aade un parmetro a la sesin.
	 * 
	 * @param name
	 *            El nombre del parmetro.
	 * @param value
	 *            El valor del parmetro.
	 */
	public void addSessionParameter(String name, Object value);

	/**
	 * Obtiene un parmetro de la sesin.
	 * 
	 * @param name
	 *            El nombre del parmetro.
	 * @return El valor del parmetro.
	 */
	public Object getSessionParameter(String name);

	/**
	 * Devuelve la versin regionalizada de un String.</p>
	 * 
	 * Debe existir un bundle con el mismo nombre que este comando, o se
	 * generar una excepcin.</p>
	 * 
	 * @param string
	 *            El String.
	 * @return La versin regionalizada del String.
	 */
	public String localize(String string);

	/**
	 * Ejecuta la lgica del comando.
	 * 
	 * @return {@link edu.uoc.myst.auxiliary.Constants#SUCCESS
	 *         Constants.SUCCESS} si el resultado es correcto, o
	 *         {@link edu.uoc.myst.auxiliary.Constants#FAIL Constants.FAIL} si
	 *         es incorrecto
	 * @throws Exception
	 */
	public String execute() throws MystException, Exception;

}
