package edu.uoc.myst.configuration;

import java.net.URL;
import java.text.DateFormat;
import java.util.logging.Logger;

import javax.servlet.ServletConfig;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import edu.uoc.myst.auxiliary.Constants;
import edu.uoc.myst.configuration.xml.MystConfiguration;
import edu.uoc.myst.international.I18N;

/**
 * Clase que gestiona la configuracin de la aplicacin usuaria del framework
 * Myst, as como los parmetros recibidos a travs del archivo de configuracin
 * de la aplicacin web.
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class ConfigurationManager {
	static Logger logger = Logger.getLogger(ConfigurationManager.class
			.getName());

	/**
	 * Mtodo que carga en memoria el archivo XML de configuracin de la
	 * aplicacin usuaria de Myst, y asigna los tipos de formatos a usar para
	 * fecha y hora.</p>
	 * 
	 * Estos datos pueden especificarse en el archivo web.xml de la siguiente
	 * manera:
	 * 
	 * <pre>
	 * &lt;servlet>
	 *    ...
	 *   &lt;init-param>
	 *      &lt;param-name>app-file&lt;/param-name>
	 *      &lt;param-value>ruta&lt;/param-value>
	 *   &lt;/init-param>
	 *   &lt;init-param>
	 *      &lt;param-name>date-style&lt;/param-name>
	 *      &lt;param-value>valor&lt;/param-value>
	 *   &lt;/init-param>
	 *   &lt;init-param>
	 *      &lt;param-name>time-style&lt;/param-name>
	 *      &lt;param-value>valor&lt;/param-value>
	 *   &lt;/init-param>
	 *   ...
	 * &lt;/servlet>
	 * </pre>
	 * 
	 * Donde "valor" es cualquier timpo de formato aceptado por la clase
	 * {@link DateFormat} (SHORT, MEDIUM, LONG o FULL).</p>
	 * 
	 * El archivo XML contiene los comandos, formularios, vistas y errores
	 * definidos por la aplicacin, debiendo ser conforme al esquema XML del
	 * framework Myst.</p>
	 * 
	 * 
	 * @param servletConfig
	 *            La configuracin del servlet.
	 * @return El objeto con la configuracin de la aplicacin.
	 * @throws Exception
	 */
	public static MystConfiguration load(ServletConfig servletConfig)
			throws Exception {
		String applicationFile = servletConfig
				.getInitParameter(Constants.CONFIG_FILE_PARAMETER);
		String dateStyle = servletConfig
				.getInitParameter(Constants.CONFIG_DATE_STYLE);
		String timeStyle = servletConfig
				.getInitParameter(Constants.CONFIG_TIME_STYLE);

		if (dateStyle != null) {
			// Se asigna el nuevo tipo de formato para las fechas.
			dateStyle = dateStyle.toUpperCase();
			if (dateStyle.equals("SHORT")) {
				I18N.setDateStyle(DateFormat.SHORT);
			} else if (dateStyle.equals("LONG")) {
				I18N.setDateStyle(DateFormat.LONG);
			} else if (dateStyle.equals("FULL")) {
				I18N.setDateStyle(DateFormat.FULL);
			}
		}

		if (timeStyle != null) {
			// Se asigna el nuevo tipo de formato para las horas.
			timeStyle = timeStyle.toUpperCase();
			if (timeStyle.equals("SHORT")) {
				I18N.setTimeStyle(DateFormat.SHORT);
			} else if (timeStyle.equals("LONG")) {
				I18N.setTimeStyle(DateFormat.LONG);
			} else if (timeStyle.equals("FULL")) {
				I18N.setTimeStyle(DateFormat.FULL);
			}
		}

		if (applicationFile == null) {
			// Se utiliza el archivo de configuracin preestablecido.
			logger.info("Archivo de configuracin no especificado. Usando preestablecido");
			applicationFile = Constants.CONFIG_FILE;
		}

		/*
		 * Se valida el archivo de configuracin, y se devuelve cargado en
		 * memoria.
		 */
		validateXML(applicationFile);
		return loadConfigFile(applicationFile);
	}

	/**
	 * Mtodo que valida un archivo XML de configuracin de una aplicacin
	 * contra el esquema del framework Myst
	 * 
	 * @param applicationFile
	 *            El archivo XML.
	 * @throws Exception
	 */
	private static void validateXML(String applicationFile) throws Exception {

		// Se instancia la fbrica de creacin de esquemas
		SchemaFactory factory = SchemaFactory
				.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

		// Se obtiene la URL de acceso al esquema de validacin
		URL xsd = Thread.currentThread().getContextClassLoader()
				.getResource(Constants.SCHEMA);

		// Se crea el esquema de validacin, y un validador para el mismo.
		Schema schema = factory.newSchema(xsd);
		Validator validator = schema.newValidator();
		logger.info("Validando archivo de configuracin " + applicationFile
				+ " contra esquema " + xsd);
		try {
			// Se intenta validar el archivo de configuracin
			validator.validate(new StreamSource(Thread.currentThread()
					.getContextClassLoader()
					.getResourceAsStream(applicationFile)));
			
		} catch (Exception e) {
			// En caso de error se  suspende la aplicacin.
			logger.info("El archivo de configuracin " + applicationFile
					+ " no es vlido");
			throw e;
		}
	}

	/**
	 * Mtodo que carga el archivo de configuracin especificado en memoria,
	 * utilizando para ello la API JAXB.
	 * 
	 * @param applicationFile
	 *            el archivo de configuracin.
	 * @return El objeto con la configuracin de la aplicacin.
	 * @throws JAXBException
	 */
	private static MystConfiguration loadConfigFile(String applicationFile)
			throws JAXBException {
		logger.info("Cargando archivo de configuracin.");
		
		// Se crea el acceso a la API JAXB
		JAXBContext context = JAXBContext.newInstance(MystConfiguration.class
				.getPackage().getName(), MystConfiguration.class
				.getClassLoader());
		
		/* Se instancia un Unmarshaller, que permite convertir los datos 
		 * XML en un rbol y alojarlos en memoria.
		 */
		Unmarshaller unmarshaller = context.createUnmarshaller();
		
		// Se crea el acceso al archivo de onfiguracin
		Source source = new StreamSource(Thread.currentThread()
				.getContextClassLoader().getResourceAsStream(applicationFile));
		
		// Se convierte el archivo de configuracin en un rbol
		JAXBElement<MystConfiguration> root = unmarshaller.unmarshal(source,
				MystConfiguration.class);
		
		// Se obtiene la raz del rbol, que da acceso a toda la estructura.
		MystConfiguration applicationConfiguration = root.getValue();
		return applicationConfiguration;
	}
}
