package edu.uoc.myst.core;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import javax.servlet.ServletException;

import edu.uoc.myst.auxiliary.FormTools;
import edu.uoc.myst.auxiliary.Constants;
import edu.uoc.myst.configuration.xml.ErrorData;
import edu.uoc.myst.configuration.xml.FilterData;
import edu.uoc.myst.configuration.xml.FilterNameData;
import edu.uoc.myst.configuration.xml.FilterNameMapper;
import edu.uoc.myst.configuration.xml.MystConfiguration;
import edu.uoc.myst.configuration.xml.CommandData;
import edu.uoc.myst.configuration.xml.FormData;
import edu.uoc.myst.configuration.xml.ViewData;
import edu.uoc.myst.exception.FilterException;
import edu.uoc.myst.exception.FormLoadException;
import edu.uoc.myst.exception.MystException;
import edu.uoc.myst.exception.NoGlobalHandlerException;
import edu.uoc.myst.exception.NoViewException;
import edu.uoc.myst.exception.NotSuchCommandException;
import edu.uoc.myst.exception.NotSuchFormException;
import edu.uoc.myst.exception.handler.ExceptionHandler;
import edu.uoc.myst.view.RawView;
import edu.uoc.myst.view.View;
import edu.uoc.myst.view.ViewHelper;

/**
 * Despachador o controlador de aplicaciones del framework Myst.</p>
 * 
 * Recibe de forma sucesiva las peticiones HTTP a travs del controlador
 * frontal. Utilizando la configuracin de la aplicacin decide qu comando debe
 * gestionar cada peticin, qu formulario lleva asociado, y si utiliza filtros
 * antes y/o despus de ejecutar el comando. Instancia el comando, le enva la
 * peticin, y en funcin de su respuesta resuelve qu vista debe mostrarse al
 * cliente, y la genera.</p>
 * 
 * En caso de acontecer excepciones, tambin decide a travs de la configuracin
 * qu gestor de excepciones y qu vista deben utilizarse para tratarlas.</p>
 * 
 * Implementa los patrones ApplicationController y Singleton.</p>
 * 
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class ApplicationController {

	static Logger logger = Logger.getLogger(FrontController.class.getName());

	/**
	 * La configuracin de la aplicacin.
	 */
	private MystConfiguration applicationConfiguration;

	/**
	 * Mtodo de creacin. Asigna la configuracin de la aplicacin.</p>
	 * 
	 * @param applicationConfiguration
	 *            La configuracin de la aplicacin.
	 */
	public ApplicationController(MystConfiguration applicationConfiguration) {
		this.applicationConfiguration = applicationConfiguration;
	}

	/**
	 * Mtodo que procesa una peticin HTTP recibida travs del objeto que
	 * recoge su contexto.</p>
	 * 
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @throws ServletException
	 * @throws IOException
	 */
	public void process(ContextObject contextObject) throws ServletException,
			IOException {

		CommandData commandData = null;
		Object form = null;
		String result = null;

		try {

			// Se obtiene el comando a travs de la peticin
			commandData = getCommand(contextObject);

			// Se obtiene el formulario asociado al comando, si lo usa
			form = getForm(commandData.getForm(), contextObject);

			// Aplicar filtros de entrada
			preFilters(commandData, form, contextObject);

			// Llamada al comando
			result = launchProcess(Class.forName(commandData.getClazz()), form,
					contextObject);

			// Aplicar filtros de salida
			postFilters(commandData, contextObject.getViewHelper(),
					contextObject);

			// Obtener y mostrar la vista de la peticim
			createView(commandData.getViews().get(result), commandData, result,
					contextObject);

		} catch (Throwable e) {
			handleExceptions(e, commandData, contextObject);
		}
	}

	/**
	 * Obtiene los datos del comando a travs de la peticin HTTP.</p>
	 * 
	 * Los datos que obiene son el formulario asociado, los fitlros de entrada y
	 * salida utilizados, y sus mapas de vistas y de errores asociados.</p>
	 * 
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @return Los datps del comando
	 * @throws NotSuchCommandException
	 */
	private CommandData getCommand(ContextObject contextObject)
			throws NotSuchCommandException {

		String path = getRequestPath(contextObject);
		CommandData result = applicationConfiguration.getCommands().get(path);

		if (result == null) {
			/*
			 * El comando obtenido no existe en la configuracin de la
			 * aplicacin.
			 */
			logger.info("El comando " + contextObject + " no existe.");
			NotSuchCommandException exception = new NotSuchCommandException(
					path);
			throw exception;
		}
		return result;
	}

	/**
	 * Mtodo que crea y rellena el formulario asociado a una peticin. Obtiene
	 * los datos para cargarlo del objeto de contexto.
	 * <p/>
	 * 
	 * En caso de que defina un mtodo para asignar el {@link ContextObject},
	 * tambin lo hace.
	 * <p/>
	 * 
	 * @param formData
	 *            El tipo del formulario.
	 * @param contextObject
	 *            el objeto de contexto.
	 * @return El formulario cargado.
	 * @throws Exception
	 */
	private Object getForm(String formName, ContextObject contextObject)
			throws Exception {

		Object form = null;
		FormData formData = null;

		if (formName != null) {
			// El comando usa un formulario
			formData = applicationConfiguration.getForms().get(formName);

			if (formData == null) {
				// El formulario no existe en la configuracin de la aplicacin.
				logger.info("El formulario "
						+ formName
						+ " debe ser correctamente definido y asignado a su comando en el archivo de configuracin.");
				NotSuchFormException exception = new NotSuchFormException(
						formName);
				throw exception;
			}
		} else {
			// El comando no usa un formulario.
			return null;
		}

		// Se trata de instanciar el formulario.
		Class<?> cls;
		try {
			cls = Class.forName(formData.getClazz());
			form = cls.newInstance();
			logger.info("Creado formulario de clase: " + cls.getName());

		} catch (ClassNotFoundException e) {
			/*
			 * No hay una clase que implemente el formulario. Se lanza una
			 * excepcin y se detiene el comando.
			 */
			logger.info("La clase " + formData.getClazz()
					+ " no existe en la aplicacin.");
			throw e;

		} catch (Exception e) {
			/*
			 * No se puede instanciar la clase del formulario. Se lanza una
			 * excepcin y se detiene el comando.
			 */
			logger.info("El mtodo de creacin de la clase "
					+ formData.getClazz() + " no es accesible.");
			throw e;
		}

		// Se intenta cargar el formulario
		logger.info("Cargando parmetros en formulario " + formData.getClazz());
		FormTools.mapToForm(form, contextObject.getRequest().getParameterMap(),
				contextObject);
		if (!contextObject.getExceptions().isEmpty()) {
			/*
			 * Han sucedido excepciones durante la carga del formulario. Se
			 * lanza una excepcin y se detiene el comando.
			 */
			throw new FormLoadException(form.getClass().getName());
		}

		// Si el formulario usa el objeto de contexto, se le asigna.
		try {
			logger.info("Asignando el contexto al comando " + cls.getName());
			Method method = cls.getMethod("setContextObject",
					ContextObject.class);
			method.invoke(form, contextObject);

		} catch (Exception e) {
			// El formulario no usa el contexto, se prosigue la ejecucin.
			logger.info("La clase " + form.getClass().getName()
					+ " no utiliza un ContextObject.");
		}
		return form;
	}

	/**
	 * Mtodo que ejecuta un comando o un filtro.</p>
	 * 
	 * Utilizando reflexin e inversin de control tratar de asignarle un
	 * formulario (o ViewHelper, para los filtros de salida), el contexto de la
	 * peticin HTTP y un ViewHelper (para los comandos y los filtros de
	 * entrada).</p>
	 * 
	 * @param cls
	 *            La clase del filtro o comando.
	 * @param form
	 *            El formulario o ViewHelper de entrada.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @return El resultado de la invocacin del filtro o comando.
	 * @throws Throwable 
	 */
	private String launchProcess(Class<?> cls, Object form,
			ContextObject contextObject) throws Throwable {

		Method method = null;
		Class<?>[] formBean = new Class[1];
		formBean[0] = Object.class;
		Object command = null;
		ViewHelper viewHelper = null;
		String result = null;

		// Carga el comando
		command = cls.newInstance();

		if (form != null) {
			// Asigna el formulario al comando
			logger.info("Asignando formulario " + form.getClass().getName()
					+ " a comando " + cls.getName());
			method = cls.getMethod("setFormBean", formBean);
			method.invoke(command, form);
		}

		// Asigna el objeto ViewHelper de ayuda, en caso de utilizarlo
		try {
			// Asigna el objeto de ayuda
			logger.info("Asignando helper a comando " + cls.getName());
			method = cls.getMethod("setViewHelper", ViewHelper.class);
			viewHelper = new ViewHelper(contextObject);
			method.invoke(command, viewHelper);

		} catch (NoSuchMethodException e) {
			// La clase no utiliza un objeto de ayuda, se prosigue la ejecucin.
			logger.info("La clase " + cls.getName()
					+ " no define un ViewHelper.");
		}

		// Asigna el contexto HTTP, en caso de utilizarlo.
		try {
			logger.info("Asignando el contexto al comando " + cls.getName());
			method = cls.getMethod("setContextObject", ContextObject.class);
			method.invoke(command, contextObject);

		} catch (NoSuchMethodException e) {
			// La clase no utiliza un objeto de ayuda, se prosigue la ejecucin.
			logger.info("La clase " + cls.getName()
					+ " no utiliza el contexto.");
		}

		// Se ejecuta el comando y se obtiene la respuesta.
		try {
			Class<?> noparams[] = {};
			logger.info("Inicio de ejecucin de accin.");
			method = cls.getDeclaredMethod("execute", noparams);
			result = (String) method.invoke(command, (Object[]) null);
			logger.info("Fin de ejecucin del comando " + cls.getName()
					+ ", con resultado " + result);
			return result;
		} catch (InvocationTargetException e) {
			// Se lanza la excepcin que caus el error.
			throw e.getCause();
		}

	}

	/**
	 * Mtodo que se encarga de confirmar u obtener la vista asociada a un
	 * comando y un resultado, de instanciarla y de solicitar su
	 * presentacin.</p>
	 * 
	 * @param viewData
	 *            La vista.
	 * @param commandData
	 *            El comando.
	 * @param result
	 *            El resultado.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @throws Exception
	 */
	private void createView(ViewData viewData, CommandData commandData,
			String result, ContextObject contextObject) throws Exception {

		View view = null;

		if (viewData == null) {
			// Si el comando no define una vista, puede ser global.
			logger.info("El comando " + commandData.getClazz()
					+ " con resultado " + result
					+ " no tiene una vista asociada.");

			if (applicationConfiguration.getGlobalViews() != null) {
				// Se trata de obtener la vista globlal
				viewData = applicationConfiguration.getGlobalViews()
						.get(result);
			}
		}
		if (viewData == null) {
			/*
			 * No existen vistas definidas para el comando. Se lanza una
			 * excepcin y se interrumpe el comando.
			 */
			logger.info("El resultado " + result + " del comando "
					+ commandData.getName()
					+ " no tiene ninguna vista local o global asociada");
			NoViewException exception = new NoViewException(result,
					commandData.getName());
			throw exception;
		}

		// Se instancia la vista
		@SuppressWarnings("unchecked")
		Class<View> viewClass = (Class<View>) Class
				.forName(viewData.getClazz());
		view = viewClass.newInstance();

		// Se cargan los parmetros de la vista y se muestra.
		showView(view, viewData.getParameters(), contextObject);

	}

	/**
	 * Mtodo que carga los parmetros de la vista en la vista actual, y que
	 * solicita su presentacin.</p>
	 * 
	 * @param view
	 *            La vista.
	 * @param parameters
	 *            Los parmetros de la vista.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @throws Exception
	 * @throws MystException
	 */
	private void showView(View view, Map<String, String> parameters,
			ContextObject contextObject) throws Exception, MystException {

		/*
		 * Carga los parmetros de la vista en un nico mapa, para poderlos
		 * tratar.
		 */
		Map<String, String[]> aux = new HashMap<String, String[]>();
		String[] value = new String[1];
		for (Entry<String, String> entry : parameters.entrySet()) {
			value[0] = entry.getValue();
			aux.put(entry.getKey(), value);
			logger.info("Parmetro: " + entry.getKey() + " - Valor: "
					+ value[0]);
		}

		// Se carga el mapa de parmetros en la vista con FomTools.
		logger.info("Carga de los parmetros de la vista.");
		List<Throwable> currentExceptions = contextObject.getExceptions();
		FormTools.mapToForm(view, aux, contextObject);

		if (currentExceptions.size() != contextObject.getExceptions().size()) {
			/*
			 * Han sucedido errores en la carga. Se lenza una excepcin y se
			 * detiene el comando.
			 */
			throw new FormLoadException(view.getClass().getName());
		}

		// Se muestra la vista
		view.show(contextObject.getViewHelper(), contextObject);
	}

	/**
	 * Mtodo que se encarga de obtener el gestor de excepciones correspondiente
	 * al comando y error sucedidos, de instanciarle y de enviar el resultado
	 * del tratamiento a la vista adecuada.</p>
	 * 
	 * @param e2
	 *            La ltima excepcin ocurrida.
	 * @param commandData
	 *            El comando.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @throws ServletException
	 * @throws IOException
	 */
	private void handleExceptions(Throwable exception, CommandData commandData,
			ContextObject contextObject) throws ServletException, IOException {

		ExceptionHandler handler;
		View view;
		Map<String, ErrorData> errorMap = null;
		ErrorData errorData = null;

		if (commandData != null) {
			// Se obtienen los gestores de errores para la peticin.
			errorMap = commandData.getErrors();
		}
		if (errorMap != null) {
			ErrorData aux = null;
			for (Entry<String, ErrorData> entry : errorMap.entrySet()) {
				// Se busca el gestor adecuado para tratar la peticin.
				aux = entry.getValue();
				try {
					if (Class.forName(aux.getType()).isInstance(exception)) {
						errorData = aux;
					}
				} catch (Exception e) {
					/*
					 * La clase del error no existe, se aade una nueva
					 * excepcin y se rposigue la gestin de errores.
					 */
					contextObject.addException(e);
				}
			}
		}

		if (errorData == null) {
			/*
			 * Si el comando no define un gestor de errores, puede ser global, o
			 * puede ser que el error no disponga de gestores especficos.
			 */
			if (commandData != null) {
				logger.info("El comando " + commandData.getClazz()
						+ " con excepcin " + exception
						+ " no tiene un gestor asociado.");
			}
			errorMap = applicationConfiguration.getGlobalErrors();
			if (errorMap != null) {
				// Existen gestores de errores globales.
				ErrorData aux = null;
				for (Entry<String, ErrorData> entry : errorMap.entrySet()) {
					/*
					 * Se recorren los errores globales hasta encontrar el
					 * primero cuya clase sea superclase del error recibido.
					 */
					aux = entry.getValue();
					try {
						if (Class.forName(aux.getType()).isInstance(exception)) {
							errorData = aux;
						}
					} catch (Exception e) {
						// La clase del error no tiene un gestor asociado.
						contextObject.addException(e);
					}
				}
			}
		}

		try {
			if (errorData == null) {
				/*
				 * No hay ningn gestor definido para el error, se lanza una
				 * excepcin.
				 */
				logger.info("La excepcin " + exception
						+ " no tiene ningn gestor global asociado.");
				throw new NoGlobalHandlerException(exception.getClass()
						.getName());

			} else {
				/*
				 * Hay un gestor asociado al error, se instacia el gestor de
				 * errores.
				 */
				@SuppressWarnings("unchecked")
				Class<ExceptionHandler> exceptionHandler = (Class<ExceptionHandler>) Class
						.forName(errorData.getClazz());
				handler = exceptionHandler.newInstance();
			}

			// Se aade la ltima excepcin acontecida a la lista de errores.
			contextObject.addException(exception);

			/*
			 * Se invoca el gestor para que realice su lgica y se obtiene la
			 * vista de presentacin.
			 */
			view = handler.handle(contextObject.getExceptions(), contextObject);

			// Se invoca la vista de presentacin.
			showView(view, errorData.getParameters(), contextObject);

		} catch (Exception e) {
			/*
			 * No hay un gestor asociado al error, o la vista de gestin de
			 * errores ha fallado. Se trata de mostrar el error final
			 */
			RawView finalView = new RawView();
			if (e instanceof MystException) {
				/*
				 * Si la ltima excepcin es propia del framework, se
				 * regionaliza y muestra por pantalla.
				 */
				finalView.show(((MystException) e)
						.getLocalizedMessage(contextObject.getLocalization()),
						contextObject);
			} else {
				/*
				 * La ltima excepcin delata un fallo de la preentacin de los
				 * errores. Se mustra la traza del error.
				 */
				logger.info("Falla mostrar errores.");
				StringWriter errors = new StringWriter();
				e.printStackTrace(new PrintWriter(errors));
				finalView.show(errors.toString(), contextObject);
			}
		}
	}

	/**
	 * Mtodo que obtiene los filtros de entrada de un comando. En caso de
	 * encontrar alguno los enva a
	 * {@link #filterChain(List, Map, Object, ContextObject) filterChain} para
	 * que los aplique.</p>
	 * 
	 * @param command
	 *            El comando.
	 * @param form
	 *            El formulario asociado al comando.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @throws Throwable 
	 */
	private void preFilters(CommandData command, Object form,
			ContextObject contextObject) throws Throwable {
		FilterNameMapper filterMapper = command.getPreFilters();
		if (filterMapper != null) {
			filterChain(filterMapper.getFilter(),
					applicationConfiguration.getFilters(), form, contextObject);
		}
	}

	/**
	 * Mtodo que obtiene los filtros de salida de un comando. En caso de
	 * encontrar alguno los enva a
	 * {@link #filterChain(List, Map, Object, ContextObject) filterChain} para
	 * que los aplique.</p>
	 * 
	 * @param command
	 *            El comando.
	 * @param viewHelper
	 *            El ViewHelper asociado al comando.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @throws Throwable 
	 */
	private void postFilters(CommandData command, ViewHelper viewHelper,
			ContextObject contextObject) throws Throwable {
		FilterNameMapper filterMapper = command.getPostFilters();
		if (filterMapper != null) {
			filterChain(filterMapper.getFilter(),
					applicationConfiguration.getFilters(), viewHelper,
					contextObject);
		}
	}

	/**
	 * Mtodo que ejecuta una sucesin de filtros sobre un objeto que sea
	 * conforme a la definicin de un {@link Form} (formulario o
	 * ViewHelper).</p>
	 * 
	 * @param filterList
	 *            La sucesin de filtros.
	 * @param filters
	 *            El mapa con los filtros definidos en la aplicacin.
	 * @param form
	 *            El formulario o ViewHelper asociado al comando.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @throws Throwable 
	 */
	private void filterChain(List<FilterNameData> filterList,
			Map<String, FilterData> filters, Object form,
			ContextObject contextObject) throws Throwable {
		String result = Constants.SUCCESS;
		FilterNameData filter = null;

		if (!filterList.isEmpty()) {
			// Se ejecuta la sucesin de filtros
			Iterator<FilterNameData> iter = filterList.iterator();
			while (iter.hasNext() && result.equals(Constants.SUCCESS)) {
				filter = iter.next();
				result = launchFilter(filter, filters, form, contextObject);
			}
		}
		if (result.equals(Constants.FAIL)) {
			/*
			 * Un filtro ha fallado, se lanza una excepcin y se interrumpe el
			 * comando.
			 */
			logger.info("El filtro " + filter.getName()
					+ " ha devuelto un resultado errneo.");
			throw new FilterException(filter.getName());
		}
	}

	/**
	 * Mtodo que lanza un filtro sobre un objeto que sea conforme a la
	 * definicin de un {@link Form} (formulario o ViewHelper).</p>
	 * 
	 * @param filterName
	 *            El nombre del filtro.
	 * @param filters
	 *            El mapa con los filtros definidos en la aplicacin.
	 * @param form
	 *            El formulario o ViewHelper asociado al comando.
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 * @return El resultado de aplicar el filtro.
	 * @throws Throwable 
	 */
	private String launchFilter(FilterNameData filterName,
			Map<String, FilterData> filters, Object form,
			ContextObject contextObject) throws Throwable {

		String filter = null;
		String result = Constants.FAIL;
		FilterData applyFilter = filters.get(filterName.getName());

		try {
			logger.info("Aplicando filtro: " + filterName.getName());
			if (applyFilter == null) {
				/*
				 * El filtro no existe en el archivo de configuracin. Se lanza
				 * una excepcin y se interrumpe el comando.
				 */
				logger.info("El filtro " + filterName.getName() + " no existe.");
				NotSuchCommandException exception = new NotSuchCommandException(
						filterName.getName());
				throw exception;
			}
			filter = applyFilter.getClazz();
			result = launchProcess(Class.forName(filter), form, contextObject);
			logger.info("Fin de ejecucin del filtro " + filterName.getName()
					+ ", con resultado " + result);

		} catch (ClassNotFoundException e) {
			/*
			 * La clase del filtro no est implementada en la aplicacin. Se
			 * lanza una excepcin y se interrumpe el comando.
			 */
			logger.info("La clase " + filter + " no existe en la aplicacin.");
			throw e;
		}
		return result;
	}

	/**
	 * Obtiene el nombre del comando en el path de la peticin HTTP.
	 * 
	 * @param contextObject
	 *            El contexto de la peticin.
	 * @return El nombre del comando.
	 */
	private String getRequestPath(ContextObject contextObject) {

		// Se obtiene el nombre del comando de la ruta de la peticin.
		String path = contextObject.getRequest().getPathInfo();
		logger.info("Peticin recibida de " + path);

		if ((path == null) || (path.length() <= 0)) {
			/*
			 * Si la ruta es nula o est vaca, se obtiene el bnombre del
			 * comando de la ruta del Servlet.
			 */
			path = contextObject.getRequest().getServletPath();
			path = path.substring(path.lastIndexOf("/") + 1,
					path.lastIndexOf("."));
			logger.info("Comando: " + path);
		}
		return (path);
	}
}
