package edu.uoc.myst.core;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import edu.uoc.myst.auxiliary.Constants;
import edu.uoc.myst.view.ViewHelper;

/**
 * Clase auxiliar que almacena los datos del contexto de la peticin, de forma
 * que las dems clases del framework Myst, y las de las aplicaciones que lo
 * utilicen, puedan acceder a los datos de la peticin de forma transparente a
 * las peticiones HTTP.</p>
 * 
 * Adems, al estar asociado a casi cualquier objeto del framework, facilita la
 * comunicacin entre las diferentes clases que lo usan, como almacenar una
 * referencia al ViewHelper para que est disponible en la generacin de vistas,
 * o la regin del contexto, para la regionalizacin.</p>
 * 
 * Implementa el patrn ContextObject.</p>
 * 
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class ContextObject {

	/**
	 * La peticin HTTP del servlet.
	 */
	private HttpServletRequest request;

	/**
	 * La respuesta HTTP del servlet.
	 */
	private HttpServletResponse response;

	/**
	 * El contexto del Servlet.
	 */
	private ServletContext servletContext;

	/**
	 * La regin asociada a la sesin.
	 */
	private Locale localization;

	/**
	 * Devuelve el {@link ViewHelper} asociado al comando actual.
	 */
	private ViewHelper viewHelper;

	/**
	 * Las excepciones sucedidas en la ejecucin del comando.
	 */
	private List<Throwable> exceptions;

	/**
	 * Mtodo de creacin.</p>
	 * 
	 * Asigna valores a los atributos del contexto de la peticin HTTP.</p>
	 * 
	 * @param request
	 *            La peticin HTTP del servlet.
	 * @param response
	 *            La respuesta HTTP del servlet.
	 * @param servletContext
	 *            El contexto del Servlet.
	 */
	public ContextObject(HttpServletRequest request,
			HttpServletResponse response, ServletContext servletContext) {
		this.request = request;
		this.response = response;
		this.servletContext = servletContext;
		this.viewHelper = null;

		/*
		 * Si la sesin no tiene asignada una regin, se le asigna la de la
		 * peticin HTTP.
		 */
		if (request.getSession().getAttribute(Constants.LOCALIZATION) == null) {
			this.setLocalization(request.getLocale());
		} else {
			// Sino se asigna la regin de la sesin a la peticin HTTP.
			this.setLocalization((Locale) request.getSession().getAttribute(
					Constants.LOCALIZATION));
		}
	}

	/**
	 * Devuelve la peticin HTTP del servlet.
	 * 
	 * @return La peticin HTTP del servlet.
	 */
	public HttpServletRequest getRequest() {
		return request;
	}

	/**
	 * Devuelve la respuesta HTTP del servlet.
	 * 
	 * @return La respuesta HTTP del servlet.
	 */
	public HttpServletResponse getResponse() {
		return response;
	}

	/**
	 * Devuelve el contexto del Servlet.
	 * 
	 * @return El contexto del Servlet.
	 */
	public ServletContext getServletContext() {
		return servletContext;
	}
	
	/**
	 * Devuelve la sesin HTTP del usuario.
	 * 
	 * @return La sesin HTTP del usuario.
	 */
	public HttpSession getSession() {
		return request.getSession();
	}

	/**
	 * Devuelve la regin de la sesin.
	 * 
	 * @return La regin de la sesin.
	 */
	public Locale getLocalization() {
		return localization;
	}

	/**
	 * Asigna una nueva regin a la peticin y a la sesin.
	 * 
	 * @param localization
	 *            La regin.
	 */
	public void setLocalization(Locale localization) {
		request.getSession().setAttribute(Constants.LOCALIZATION, localization);
		request.setAttribute(Constants.LOCALIZATION, localization);
		this.localization = localization;
	}

	/**
	 * Asigna un {@link ViewHelper} que ayudar a la presentacin del resultado
	 * del comando actual.
	 * 
	 * @param viewHelper
	 *            El viewHelper.
	 */
	public void setViewHelper(ViewHelper viewHelper) {
		this.viewHelper = viewHelper;
		request.getSession().setAttribute(Constants.VIEWHELPER, viewHelper);
	}

	/**
	 * Devuelve el {@link ViewHelper} asociado al comando actual.
	 * 
	 * @return El ViewHelper.
	 */
	public ViewHelper getViewHelper() {
		return viewHelper;
	}

	/**
	 * 
	 * Aade una excepcin acontecida durante la ejecucin de un comando.</p>
	 * 
	 * @param exception
	 *            La excepcin.
	 */
	public void addException(Throwable exception) {
		if (exceptions == null) {
			exceptions = new ArrayList<Throwable>();
		}
		exceptions.add(exception);
	}

	/**
	 * Devuelve las excepciones acontecidas durante la ejecucin de un comando
	 * que han sido guardadas en el contexto.</p>
	 * 
	 * @return Las excepciones del comando.
	 */
	public List<Throwable> getExceptions() {
		if (exceptions == null) {
			exceptions = new ArrayList<Throwable>();
		}
		return this.exceptions;
	}
}
