package edu.uoc.myst.filter;

import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import edu.uoc.myst.exception.NotFormattableException;
import edu.uoc.myst.international.I18N;
import edu.uoc.myst.view.ViewHelper;
import edu.uoc.myst.auxiliary.Constants;
import edu.uoc.myst.auxiliary.FormTools;

/**
 * Filtro de salida que regionaliza automticamente todos los datos del
 * ViewHelper asociado al comando.</p>
 * 
 * Para poder ser regionalizados, los datos deben ser tipos primitivos de Java;
 * de tipo {@link java.util.Date Date}, compatibles con la especificacin de un
 * {@link edu.uoc.myst.form.Form Form}; con datos accesibles por getters; arrays
 * de tipos primitivos, Dates o forms; o cualquier combinacin de los
 * anteriores.</p>
 * 
 * Utiliza la biblioteca de regionalizacin
 * {@link edu.uoc.myst.international.I18N I18N}.</p>
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class InternationalFormatFilter extends AbstractFilter {

	static Logger logger = Logger.getLogger(InternationalFormatFilter.class
			.getName());

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.myst.filter.Filter#execute()
	 */
	@SuppressWarnings("unchecked")
	public String execute() throws NotFormattableException {
		Map<String, Object> parametersMap = ((ViewHelper) getFormBean())
				.getParametersMap();
		Map<String, Object> result = new HashMap<String, Object>();

		if (parametersMap != null) {
			try {
				// Convierte los elementos del mapa del ViewHelper
				result = (Map<String, Object>) format(parametersMap,
						getContextObject().getLocalization());
				((ViewHelper) getFormBean()).setParametersMap(result);

			} catch (NotFormattableException e) {
				/*
				 * Si acontece una excepcin se aade a la lista y se detiene el
				 * procesamiento del comando.
				 */
				addException(e);
				throw e;
			}
		}
		return Constants.SUCCESS;
	}

	/**
	 * 
	 * Regionaliza el objeto recibido.
	 * 
	 * @param value
	 *            El objeto a regionalizar.
	 * @param locale
	 *            La regin.
	 * @return El objeto regionalizado.
	 * @throws NotFormattableException
	 */
	private Object format(Object value, Locale locale)
			throws NotFormattableException {

		if (value == null) {
			// Si el valor es null se devuelve.
			return null;

		} else {
			logger.info("Comienzo de la localizacin del objeto " + value
					+ " de tipo " + value.getClass().getName());

			if (value.getClass().isArray()) {
				// El objeto es un array
				Object[] result = new Object[Array.getLength(value)];

				for (int j = 0; j < Array.getLength(value); j++) {
					/*
					 * Se busca el primer elemento no nulo del array para ver su
					 * tipo.
					 */
					if (Array.get(value, j) != null) {
						/*
						 * Se ha encontrado una componente no nula, por lo que
						 * pueden regionalizarse las componentes.
						 */

						if (FormTools.isPrimitive(Array.get(value, j))) {
							/*
							 * La componente inspeccionada es de tipo primitivo,
							 * por lo cual tambin el Array. Se regionalizan una
							 * a una a travs de I18N.
							 */
							for (int i = j; i < Array.getLength(value); i++) {
								Array.set(result, i, I18N.formatElement(
										Array.get(value, i), locale));
							}

						} else {
							/*
							 * La componente no es primitiva. Se comprueban una
							 * a una a travs de este mismo mtodo format.
							 */
							for (int i = j; i < Array.getLength(value); i++) {
								Array.set(result, i,
										format(Array.get(value, i), locale));
							}
						}
						/*
						 * Se ha recorrido el array entero, se actualiza la
						 * variable de control al ltimo elemento.
						 */
						j = Array.getLength(value);

					} else {
						/*
						 * La componente inspeccionada es nula, se aade tal
						 * cual
						 */
						Array.set(result, j, null);
					}
				}
				return result;

			} else if (value instanceof Map<?, ?>) {
				/*
				 * El objeto es un map. Se comprueban sus componentes
				 * individualmente a travs de este mtodo format.
				 */
				Map<Object, Object> result = new HashMap<Object, Object>();
				for (Entry<?, ?> entry : ((Map<?, ?>) value).entrySet()) {
					result.put(entry.getKey(), format(entry.getValue(), locale));
				}
				return result;

			} else {
				if (FormTools.isPrimitive(value)) {
					// El objeto es un tipo primitivo. Se regionaliza.
					return I18N.formatElement(value, locale);

				} else {
					// El objeto no es regionalizable.
					logger.info("La clase " + value.getClass().getName()
							+ " no es regionalizable.");
					throw new NotFormattableException(value.getClass()
							.getName());
				}
			}
		}
	}
}
