package edu.uoc.myst.international;

import java.text.DateFormat;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.logging.Logger;

/**
 * Clase que facilita la regionalizacin del framework Myst, y de las
 * aplicaciones que lo utilizan.</p>
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class I18N {

	static Logger logger = Logger.getLogger(I18N.class.getName());

	/**
	 * Tipo de formato para fechas.
	 */
	private static int DATE_STYLE = DateFormat.DEFAULT;

	/**
	 * Tipo de formato para horas.
	 */
	private static int TIME_STYLE = DateFormat.DEFAULT;

	/**
	 * Devuelve la versin regionalizada de un objeto, a travs de un bundle y
	 * una regin.
	 * 
	 * @param bundle
	 *            El nombre del bundle.
	 * @param object
	 *            El nombre del objeto.
	 * @param localization
	 *            La regin.
	 * @return El String regionalizado.
	 */
	public static String getString(String bundle, String object,
			Locale localization) throws Exception {
		return ResourceBundle.getBundle(bundle, localization).getString(object);
	}

	/**
	 * Devuelve la versin regionalizada de un mensaje compuesto a travs de un
	 * bundle, una regin y un conjunto de argumentos.</p>
	 * 
	 * El conjunto de argumentos debe ser un array de objetos, de la siguiente
	 * manera:
	 * 
	 * <pre>
	 * Object[] messageArguments = { messages.getString(&quot;planet&quot;), new Integer(7),
	 * 		new Date() };
	 * </pre>
	 * 
	 * Y se corresponder con un mensaje en el bundle, con la siguiente
	 * estructura:
	 * 
	 * <pre>
	 * message = At {2,time,short} on {2,date,long}, \
	 * we detected {1,number,integer} spaceships on \
	 * the planet {0}.
	 * planet = Mars
	 * </pre>
	 * 
	 * De forma que el primer objeto del array sustituir el elemento marcado
	 * con 0 en el mensaje y sucesivamente, obtenindose el siguiente resultado:
	 * 
	 * <pre>
	 * At 10:16 AM on July 31, 2009, we detected 7
	 * spaceships on the planet Mars.
	 * </pre>
	 * 
	 * @param className
	 *            El nombre del bundle.
	 * @param message
	 *            El nombre del mensaje.
	 * @param messageArguments
	 *            El conjunto de argumentos.
	 * @param localization
	 *            La regin.
	 * @return El mensaje regionalizado.
	 */
	public static String getMessage(String className, String message,
			Object[] messageArguments, Locale localization) {

		// Se obtiene el acceso al bundle.
		ResourceBundle messages = ResourceBundle.getBundle(className,
				localization);

		// Se instancia el formateador de mensajes, y se define su regin.
		MessageFormat formatter = new MessageFormat("");
		formatter.setLocale(localization);

		// Se aplica el patrn de formateo, y se devuelve el mensaje formateado.
		formatter.applyPattern(messages.getString(message));
		return formatter.format(messageArguments);
	}

	/**
	 * Regionaliza un tipo de dato primitivo deJava . Fechas y nmeros sern
	 * regionalizados, mientras que el resto de tipos sern convertidos en
	 * String.
	 * 
	 * @param element
	 *            El dato.
	 * @param localization
	 *            La regin.
	 * @return El dato regionalizado.
	 */
	public static String formatElement(Object element, Locale localization) {
		if (isNumber(element)) {
			// El elemento es de tipo numrico.
			logger.info("Formatear nmero " + element);
			return getNumber(element, localization);

		} else if (Date.class.isAssignableFrom(element.getClass())) {
			// El elemento es de tipo Date o una subclase
			logger.info("Formatear fecha " + element);
			return getDateAndTime(element, localization);
		}

		// El elemento es un String
		logger.info("Formatear cadena " + element);
		return element.toString();
	}

	/**
	 * Regionaliza el nmero pasado por parmetro.
	 * 
	 * @param number
	 *            El nmero.
	 * @param localization
	 *            La regin.
	 * @return El nmero regionalizado.
	 */
	public static String getNumber(Object number, Locale localization) {
		/*
		 * El nmero se convierte previamente en un doble para evitar errores
		 * con ciertos tipos conflictivos, como los nmeros decimales.
		 */
		return NumberFormat.getNumberInstance(localization).format(
				Double.parseDouble(number.toString()));
	}

	/**
	 * Regionaliza la fecha pasada por parmetro utilizando el estilo definido
	 * en las opciones de configuracin del Servlet.
	 * 
	 * @param element
	 *            La fecha.
	 * @param localization
	 *            La regin.
	 * @return La fecha regionalizada.
	 */
	public static String getDate(Object element, Locale localization) {
		return DateFormat.getDateInstance(DATE_STYLE, localization).format(
				element);
	}

	/**
	 * Regionaliza la fecha pasada por parmetro con el formato proporcionado.
	 * 
	 * @param element
	 *            La fecha.
	 * @param localization
	 *            La regin.
	 * @param style
	 *            El formato proporcionado.
	 * @return La fecha regionalizada.
	 */
	public static String getDate(Object element, Locale localization, int style) {
		return DateFormat.getDateInstance(style, localization).format(element);
	}

	/**
	 * Regionaliza la hora pasada por parmetro utilizando el estilo definido en
	 * las opciones de configuracin del Servlet.
	 * 
	 * @param element
	 *            La hora.
	 * @param localization
	 *            La regin.
	 * @return La hora regionalizada.
	 */
	public static String getTime(Object element, Locale localization) {
		return DateFormat.getTimeInstance(TIME_STYLE, localization).format(
				element);
	}

	/**
	 * Regionaliza la hora pasada por parmetro con el formato proporcionado.
	 * 
	 * @param element
	 *            La hora.
	 * @param localization
	 *            La regin.
	 * @param style
	 *            El formato proporcionado.
	 * @return La hora regionalizada.
	 */
	public static String getTime(Object element, Locale localization, int style) {
		return DateFormat.getTimeInstance(style, localization).format(element);
	}

	/**
	 * Regionaliza la fecha y hora pasadas por parmetro utilizando el estilo
	 * definido en las opciones de configuracin del Servlet.
	 * 
	 * @param element
	 *            La fecha y hora.
	 * @param localization
	 *            La regin.
	 * @return La fecha y la hora regionalizadas.
	 */
	public static String getDateAndTime(Object element, Locale localization) {
		return DateFormat.getDateTimeInstance(DATE_STYLE, TIME_STYLE,
				localization).format(element);
	}

	/**
	 * Regionaliza la fecha y hora pasadas por parmetro con los formatos
	 * proporcionado.
	 * 
	 * @param element
	 *            La fecha y hora.
	 * @param localization
	 *            La regin.
	 * @param dateStyle
	 *            El formato de fecha proporcionado.
	 * @param timeStyle
	 *            El formato de hora proporcionado.
	 * @return La fecha y la hora regionalizadas.
	 */
	public static String getDateAndTime(Object element, Locale localization,
			int dateStyle, int timeStyle) {
		return DateFormat.getDateTimeInstance(dateStyle, timeStyle,
				localization).format(element);
	}

	/**
	 * Asigna el tipo de formato para fechas.
	 * 
	 * @param level
	 *            El tipo de formato.
	 */
	public static void setDateStyle(int level) {
		DATE_STYLE = level;
	}

	/**
	 * Asigna el tipo de formato para horas.
	 * 
	 * @param level
	 *            El tipo de formato.
	 */
	public static void setTimeStyle(int level) {
		TIME_STYLE = level;
	}

	/**
	 * Comprueba si el dato pasado por parmero es numrico.
	 * 
	 * @param value
	 *            El dato.
	 * @return true si es numrico, false si no lo es
	 */
	private static boolean isNumber(Object value) {
		try {
			Double.parseDouble(value.toString());
		} catch (Exception e) {
			// Si el objeto no puede ser convertido a un doble, no es un nmero.
			return false;
		}
		return true;
	}
}
