package edu.uoc.myst.view;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import edu.uoc.myst.auxiliary.FormTools;
import edu.uoc.myst.core.ContextObject;

/**
 * Objeto que facilita el acceso a la informacin del negocio desde pginas JSP.
 * Permite guardar los elementos deseados de la lgica en un mapa, a los que se
 * accede cmodamente desde las pginas JSP con el tag form de la biblioteca
 * Myst.</p>
 * 
 * Al almacenar los elementos, convierte los objetos tipo {@link Collection} en
 * arrays, y los objetos tipo {@link edu.uoc.myst.form.Form Form} en mapas, a
 * travs de los mtodos de clase de {@link FormTools}. Esta operacin se
 * realiza de forma recursiva sobretodas las componentes de los elementos.</p>
 * 
 * Implementa el patrn ViewHelper.</p>
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * @see edu.uoc.myst.view.taglib.FormTag FormTag
 * 
 */
public class ViewHelper {

	static Logger logger = Logger.getLogger(ViewHelper.class.getName());

	/**
	 * Mapa que almacena los elementos de la lgica de negocio.
	 */
	private Map<String, Object> parametersMap;

	/**
	 * El objeto con el contexto de la peticin.
	 */
	private ContextObject contextObject;

	/**
	 * Mtodo de creacin. Asigna el contexto de la peticin, y a la vez asigna
	 * el ViewHelper actual al contexto.</p>
	 * 
	 * @param contextObject
	 *            el contexto de la peticin HTTP.
	 */
	public ViewHelper(ContextObject contextObject) {
		this.contextObject = contextObject;
		contextObject.setViewHelper(this);
	}

	/**
	 * Obtiene el mapa de los elementos de la lgica de negocio.
	 * 
	 * @return El mapa.
	 */
	public Map<String, Object> getParametersMap() {
		return parametersMap;
	}

	/**
	 * Asigna el contexto de la peticin HTTP.
	 * 
	 * @param contextObject
	 *            El contexto de la peticin HTTP.
	 */
	public void setContextObject(ContextObject contextObject) {
		this.contextObject = contextObject;
	}

	/**
	 * Devuelve el contexto de la peticin HTTP.
	 * 
	 * @return El contexto de la peticin HTTP.
	 */
	public ContextObject getContextObject() {
		return this.contextObject;
	}

	/**
	 * Asigna el mapa de los elementos de la lgica de negocio.
	 * 
	 * @param parametersMap
	 *            El mapa.
	 */
	public void setParametersMap(Map<String, Object> parametersMap) {
		this.parametersMap = parametersMap;
	}

	/**
	 * Guarda un elemento en el mapa de los elementos de la lgica de negocio.
	 * 
	 * @param name
	 *            El nombre del elemento.
	 * @param value
	 *            El valor del elemento.
	 */
	public void addParameter(String name, Object value) {
		if (parametersMap == null) {
			parametersMap = new HashMap<String, Object>();
		}
		try {
			parametersMap.put(name, transform(value));
		} catch (Exception e) {
			/*
			 * Si acontece una excepcin se aade a la lista y se prosigue la
			 * ejecucin del comando.
			 */
			contextObject.addException(e);
		}
	}

	/**
	 * Convierte los objetos de forma recursiva al formato de almacenamiento
	 * interno definido en la clase {@link ViewHelper}
	 * 
	 * @param value
	 *            El objeto a convertir.
	 * @return El objeto convertido.
	 * @throws Exception
	 */
	private Object transform(Object value) throws Exception {

		if (value == null) {
			// Si el valor es nulo se devuelve sin ms.
			return null;

		} else if (FormTools.isPrimitive(value)) {
			// El objeto es de tipo primitivo, se devuelve tal cual.
			return value;

		} else if (value.getClass().isArray()) {
			/*
			 * El objeto es un array. Hay que observar el tipo de sus elementos
			 * para saber cmo procesarlos.
			 */
			logger.info("El objeto es un array");
			Object[] result = new Object[Array.getLength(value)];

			for (int j = 0; j < Array.getLength(value); j++) {
				/*
				 * Se busca el primer elemento no nulo del array para ver su
				 * tipo.
				 */
				if (Array.get(value, j) != null) {
					/*
					 * Se ha encontrado un objeto no nulo, puede saberse su // *
					 * tipo.
					 */
					if (FormTools.isForm(Array.get(value, j))) {
						/*
						 * Las componentes del array son forms. Se convierten en
						 * maps y se examinan de forma recursiva.
						 */
						Map<String, Object> map = new HashMap<String, Object>();
						for (int i = j; i < Array.getLength(value); i++) {
							map = FormTools.formToMap(Array.get(value, i));
							Array.set(result, i, transform(map));
						}

					} else {
						/*
						 * Las componentes del array no son forms. Se examinan
						 * de forma recursiva.
						 */
						for (int i = j; i < Array.getLength(value); i++) {
							Array.set(result, i, transform(Array.get(value, i)));
						}
					}

					/*
					 * Se ha recorrido el array entero, se actualiza la variable
					 * de control.
					 */
					j = Array.getLength(value);

				} else {
					// La componente es nula, se aade tal cual.
					Array.set(result, j, null);
				}
			}

			return result;

		} else if (value instanceof Collection<?>) {
			// El objeto es una coleccin. Se convierte en array.
			logger.info("Convertir coleccin en array.");
			return transform(((Collection<?>) value).toArray());

		} else if (value instanceof Map<?, ?>) {
			/*
			 * El objeto es un mapa, se comprueban sus elementos de forma
			 * individual.
			 */
			logger.info("El objeto es un map.");
			Map<Object, Object> result = new HashMap<Object, Object>();
			Object nombre;
			Object valor;
			for (Entry<?, ?> entry : ((Map<?, ?>) value).entrySet()) {
				nombre = entry.getKey();
				valor = entry.getValue();
				result.put(nombre, transform(valor));
			}
			return result;

		} else if (FormTools.isForm(value)) {
			// El objeto es un form. Se convierte en un mapa.
			logger.info("El objeto es un form.");
			return transform(FormTools.formToMap(value));

		} else {
			/*
			 * El objeto es de un tipo no controlado por la lgica, se devuelve
			 * sin ms.
			 */
			logger.info("El objeto " + value + "es de tipo desconocido.");
			return value;
		}
	}
}
