package edu.uoc.myst.view.taglib;

import java.io.IOException;
import java.util.Locale;
import java.util.logging.Logger;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import edu.uoc.myst.auxiliary.Constants;
import edu.uoc.myst.international.I18N;

/**
 * Tag que permite regionalizar una cadena. La sintaxis de uso en una peticin
 * JSP es la siguiente:</p>
 * 
 * &lt;myst:i18n message="cadena" bundle="nombreDelBundle"/></p>
 * 
 * Donde cadena es el nombre del objeto que se quiere regionalizar, y
 * nombreDelBundle el bundle donde se encuentra dicho String.</p>
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * 
 */
public class I18NTag extends TagSupport {

	private static final long serialVersionUID = -2883933402713294940L;
	static Logger logger = Logger.getLogger(I18NTag.class.getName());

	/**
	 * Objeto a regionalizar.
	 */
	private String message;

	/**
	 * Bundle donde se encuentran las versiones regionalizadas del objeto.
	 */
	private String bundle;

	/**
	 * Recoge el nombre del objeto definido en el parmetro "message" de la
	 * peticin JSP.
	 * 
	 * @param message
	 *            El nombre del objeto.
	 */
	public void setMessage(String message) {
		this.message = message;
	}

	/**
	 * Recoge la clase del bundle que se utilizar para la regionalizacin del
	 * objeto. Esta clase se define en el parmetro "bundle" de la peticin JSP.
	 * 
	 * @param bundle
	 *            La clase del bundle
	 */
	public void setBundle(String bundle) {
		this.bundle = bundle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.jsp.tagext.TagSupport#doStartTag()
	 */
	public int doStartTag() throws JspException {
		try {
			String result;
			try {
				result = getMessage();

			} catch (Exception e) {
				/*
				 * No se ha podido acceder al bundle, se mustra como mensaje el
				 * nombre del objeto a regionalizar.
				 */
				logger.info("Imposible acceder al bundle " + bundle);
				result = message;
			}
			// Se intenta escribir el mensaje en la pgina.
			pageContext.getOut().write(result);

		} catch (IOException e) {
			// Si sucede un error se lanza uan excepcin JSP.
			throw new JspException(e.getLocalizedMessage(), e);
		}
		return TagSupport.SKIP_BODY;
	}

	/**
	 * Obtiene la versin regionalizada del objeto definido en el parmetro
	 * "message" de la peticin JSP.
	 * 
	 * @return La versin regionalizada del objeto.
	 * @throws Exception
	 */
	private String getMessage() throws Exception {
		Locale locale;
		if (pageContext.getSession().getAttribute(Constants.LOCALIZATION) == null) {
			/*
			 * Si no hay una regin definida en la sesin se utiliza la de la
			 * peticin.
			 */
			locale = pageContext.getRequest().getLocale();
		} else {
			locale = (Locale) pageContext.getSession().getAttribute(
					Constants.LOCALIZATION);
		}
		return I18N.getString(bundle, message, locale);
	}

}
