package edu.uoc.myst.view.taglib;

import java.util.Locale;
import java.util.logging.Logger;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.*;

import edu.uoc.myst.auxiliary.Constants;
import edu.uoc.myst.auxiliary.RegExTools;
import edu.uoc.myst.exception.NoParameterException;
import edu.uoc.myst.exception.NoPathException;
import edu.uoc.myst.exception.WrongPatternException;
import edu.uoc.myst.view.ViewHelper;

/**
 * Tag que permite realizar iteraciones sobre vectores del mapa definido en el
 * objeto ViewHelper. La sintaxis de uso en una peticin JSP es la
 * siguiente:</p>
 * 
 * <pre>
 * &lt;myst:iter var="variable" element="elemento">
 * 	$&lt;variable>
 * &lt;myst:iter/>
 * </pre>
 * 
 * Donde elemento es la ruta del vector que se quiere obtener, y variable la
 * variable donde ir iterando el vector.</p>
 * 
 * @author Miguel Souto Bartolom
 * @version 1.0 13/01/2014
 * @see edu.uoc.myst.view.ViewHelper
 * 
 */
public class IterTag extends TagSupport {

	private static final long serialVersionUID = 7581585970831122828L;
	static Logger logger = Logger.getLogger(IterTag.class.getName());

	Object[] realVector;
	private int numIter;
	private int position;

	/**
	 * Objeto ViewHelper asociado al comando.
	 */
	private ViewHelper viewHelper;

	/**
	 * Variable de la iteracin.
	 */
	private String var;

	/**
	 * Ruta del vector a iterar.
	 */
	private String element;

	/**
	 * Recoge la variable de iteracin.
	 * 
	 * @param var
	 *            La variable.
	 */
	public void setVar(String var) {
		this.var = var;
	}

	/**
	 * Recoge la ruta del vector que se quiere iterar.
	 * 
	 * @param element
	 *            El nombre del vector a iterar.
	 */
	public void setElement(String element) {
		this.element = element;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.jsp.tagext.TagSupport#doStartTag()
	 */
	public int doStartTag() throws JspException {
		try {
			// Comprueba que la ruta del vector existe
			realVector = getVector();

			/*
			 * Si sucede una excepcin se regionaliza para mostrarla por
			 * pantalla y se lanza una excepcin JSP.
			 */
		} catch (WrongPatternException e) {
			throw new JspException(e.getLocalizedMessage(), e);
		} catch (NoPathException e) {
			throw new JspException(e.getLocalizedMessage(), e);
		}

		if (realVector == null) {
			/*
			 * Si la ruta es nula se genera un mensaje de error regionalizado y
			 * se detiene la ejecucin del bucle.
			 */
			NoParameterException excp = new NoParameterException(element);
			pageContext.setAttribute(var, excp
					.getLocalizedMessage((Locale) pageContext.getSession()
							.getAttribute(Constants.LOCALIZATION)));
			return SKIP_BODY;
		}

		/*
		 * Si todo ha ido bien se inicializan las variables de control para
		 * comenzar la ejecucin del bucle.
		 */
		position = 0;
		numIter = realVector.length;

		// Se asigna la primera posicin del vector a la variable.
		pageContext.setAttribute(var, (element + "[" + position + "]"));
		return EVAL_BODY_INCLUDE;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.jsp.tagext.TagSupport#doAfterBody()
	 */
	public int doAfterBody() {
		if (position < (numIter - 1)) {
			/*
			 * Mientras haya componentes en el vector se asigna su valor a la
			 * variable.
			 */
			position++;
			pageContext.setAttribute(var, (element + "[" + position + "]"));
			return EVAL_BODY_AGAIN;
		} else {
			return EVAL_PAGE;
		}
	}

	/**
	 * Obtiene el vector a iterar del mapa del ViewHelper.
	 * 
	 * @return El vector.
	 * @throws WrongPatternException
	 * @throws NoPathException
	 */
	private Object[] getVector() throws WrongPatternException, NoPathException {
		viewHelper = (ViewHelper) pageContext.getSession().getAttribute(
				Constants.VIEWHELPER);
		return (Object[]) (RegExTools.getElement(element,
				viewHelper.getParametersMap()));
	}
}
